--[[ 
	General notes: 
		- If you don't care about the return value of a function, don't
		  return anything.
		- If you don't process an event, then don't define it.
	
	The only required functions are: GetRequiredVersion and OnScriptLoad.
--]]

function GetRequiredVersion()
	return 200
end

function OnScriptLoad(processid, game, persistent)
end

function OnScriptUnload()
end

--[[
-- Called when a player has changed teams.
-- returns boolean indicating whether or not the player should change teams.
-- Note: relevant is similar to voluntary (true) or involuntary (false).
-- If relevant is true the return value is considered, if it's false then the return value is ignored.
function OnTeamChange(player, old_team, new_team, relevant)
end	

-- Called when a server command is being executed. 
-- returns boolean indicating whether or not Phasor should process the command.
 function OnServerCommand(player, command)
end

-- Called when a player without the correct password is trying to execute a server command. 
-- returns boolean indicating whether or not the command can be processed.
-- By default Phasor will block such attempts.
function OnServerCommandAttempt(player, command, password)
end
 
-- Called when a new game is starting.
function OnNewGame(map)
end
 	
 -- Called when a game is ending.
function OnGameEnd(stage)
	-- stages:
    -- 1 The game has just ended, and the ingame score card is shown.
    -- 2 The postgame scorecard is shown.
    -- 3 Players can now leave.
end
 
-- Called when a player is attempting to join.
-- returns boolean indicating whether or not the player is allowed to join.
function OnBanCheck(hash, ip)
end	
 
-- Called when a client sends its update packet. This function is called 30 times a second for every player in the server.
-- Do not process this event unless you absolutely have to, and when processing it make your code as efficient as possible 
function OnClientUpdate(player)
end
 
-- Called when a player successfully joins the game.
function OnPlayerJoin(player)
end
 
-- Called when a player quits.
function OnPlayerLeave(player)
end	
 
-- Called when a player needs to be assigned a team.
-- return the team you want the player on
function OnTeamDecision(team)
end	
 
-- Called when a player has spawned.
function OnPlayerSpawn(player, m_objectId)
end	
 
-- Called when the server has been notified on the player's spawn.
function OnPlayerSpawnEnd(player, m_objectId)
end	
 
-- Called when an object has just been created. You can modify most object settings and have it sync.
function OnObjectCreation(m_objectId)
end	
 
-- Called when an object wants to be created. You can block it.
-- returns boolean indicating whether or not the object should be created OR map id of object to create instead.
function OnObjectCreationAttempt(mapid, parentid, player)
end	
 
-- Called when an object is being assigned their spawn weapons.
-- For this to have any effect the gametype must have starting equipment set to generic.
-- returns the map id as 'weap_id' of the weapon you wish to assign.
function OnWeaponAssignment(player, owner_id, order, weap_id)

    -- nil if you don't want to change the assigned weapon.
    -- or the map id of the weapon you wish to assign
    -- or -1 if you don't want the weapon to be assigned.
end
 
-- Called when a player interacts with an object (ie stands on it)
-- returns boolean indicating whether or not to allow the interaction.
function OnObjectInteraction(player, objid, mapid)
end	
 
-- Called when the server needs to apply damage to an object.
-- returns boolean indicating whether or not to allow the damage.
function OnDamageLookup(receiver, causer, tagid)
	-- usage has changed considerably from older versions, consult the documentation.
end	
 
-- Called when the server is about to apply damage to an object.
-- returns boolean indicating whether or not to allow the damage.
function OnDamageApplication(receiving, causing, tagid, hit, backtap)
end	
 
-- Called when a player chats in the server.
-- returns allow, msg, type.  return true to allow, false to block
function OnServerChat(player, chattype, msg)

	-- type:
    -- 0 All chat
    -- 1 Team chat
    -- 2 Vehicle chat
    -- 3 Server message
    -- 4 Private server message
	-- return true, chattype, msg
end	
 
-- Called when a player is wanting to enter a vehicle.
-- returns boolean indicating whether or not they should be allowed to enter.
function OnVehicleEntry(player, veh_id, seat, mapid, relevant)
end	
 
-- Called when a player is leaving a vehicle.
-- returns boolean indicating whether or not you can stop them leaving
function OnVehicleEject(player, relevant)
end	
 
-- Called when a player is killed. Killer can be nil. victim is never nil.
function OnPlayerKill(killer, victim, mode)
	
	-- modes:
    -- 0 Killed by the server.
    -- 1 Killed by fall damage.
    -- 2 Killed by the guardians.
    -- 3 Killed by a vehicle.
    -- 4 Killed by killer
    -- 5 Betrayed by killer
    -- 6 Suicide

end	
 
-- Called when a player gets a kill streak.
function OnKillMultiplier(player, multiplier)
	
	-- Valid multipliers are:
    -- 7 Double kill
    -- 9 Triple kill
    -- 10 Killtacular
    -- 11 Killing spree
    -- 12 Running riot
	-- I think the these happen in Slayer:
    -- 16 Double kill
    -- 15 Triple kill 
    -- 14 Killtacular 
    -- 18 Killing spree
    -- 17 Running riot
end	
 
-- Called when a weapon is being reloaded. 
-- returns Boolean indicating whether or not they can reload.
function OnWeaponReload(player, objid)
end

--]]